/*
 * Copyright (c) 2022, salesforce.com, inc.
 * All rights reserved.
 * SPDX-License-Identifier: MIT
 * For full license text, see the LICENSE file in the repo root or https://opensource.org/licenses/MIT
 */
var NodeURLSearchParams = require('url').URLSearchParams;
var compiler = require('@lwc/compiler');
var getInfoFromPath = require('./module').getInfoFromPath;
module.exports = function loader(source) {
    var _a = this, resourcePath = _a.resourcePath, resourceQuery = _a.resourceQuery, getOptions = _a.getOptions;
    var _b = getOptions(), stylesheetConfig = _b.stylesheetConfig, outputConfig = _b.outputConfig, experimentalDynamicComponent = _b.experimentalDynamicComponent;
    var info;
    try {
        info = getInfoFromPath(resourcePath, process.cwd());
    }
    catch (e) {
        info = {
            name: '',
            namespace: '',
        };
    }
    var codeTransformed = source;
    // Scoped styles have paths like this: "foo.scoped.css?scoped=true"
    var scopedStyles = resourcePath.endsWith('.css') &&
        resourceQuery &&
        new NodeURLSearchParams(resourceQuery).get('scoped') === 'true';
    var code = compiler.transformSync(codeTransformed, resourcePath, {
        name: info.name,
        namespace: info.ns,
        stylesheetConfig: stylesheetConfig,
        outputConfig: outputConfig,
        experimentalDynamicComponent: experimentalDynamicComponent,
        scopedStyles: scopedStyles,
    }).code;
    return code;
};
